#!/usr/bin/perl

use strict;

#------------------------------------------------------------------------------
# Configure S-terminal:
# Read sterminal.conf, fetch updated information from setupurl.
# Configuration information includes:
#  setupurl     A url from which to obtain updated information. You may
#               use values returned by LocalInfo (see below) within the
#               url, and these will be expanded before the url is used.
#               for example: http://s-terminal.%ipdomain%/%hwaddr%.dat
#               In addition to the variables returned by LocalInfo,
#               you may also use the 'group' value defined in
#               sterminal.conf.
#  bgcolor      The background color for the login screen.
#  bgimage      A background image for the login screen.  Currently,
#               this must be 480x480.
#
#  Each of the following parameters specifies an arbitrary command
#  to run at some point during the lifetime of a session.  Note
#  that these commands will be executed on the LOCAL machine, not
#  the remote system.  Also note that a command like "xterm" may pause
#  S-terminal's execution until the xterm exits, unless you explicitly
#  put the command into the background AND disconnect its STDIN and 
#  STDOUT (e.g., "xterm > /dev/null 2>&1 &").
#
#  prelogin     Executed before the login window appears.  Will be 
#               executed within perl's backticks.
#  presession   Executed after login window disappears, but before 
#               attempting to start a session. Will be executed by expect's "system" function.
#  atsession    Executed immediately after session has been successfully
#               begun. Executed by expect's "system" function.
#  postsession  Executed after session ends, by expect's "system" function.
#  postlogin    Executed before exiting S-terminal, by perl's backticks.
#------------------------------------------------------------------------------
sub Configure {

    my $basedir = shift;
    my $vars = shift;

    my %setup = ();
    my @data = ();
    
    # Get default configuration:
    if (-e "$ENV{HOME}/.sterminal.conf") {
      open ( CONFIG, "$ENV{HOME}/.sterminal.conf");
    } else { open ( CONFIG, "<$basedir/sterminal.conf" ); }
    chomp(@data = <CONFIG>);
    close ( CONFIG );
    for my $i (@data) {
	for ($i) {
	    /^\s*([^:]*[^\s:])\s*:\s*(.*[^\s])?\s*$/ && do {
		$setup{$1} = $2?$2:"";
	    }
	}
    }
    
    # Add group name from config file to vars:
    $vars->{group} = $setup{group};

    # Fetch updated configuration:
    if ( $setup{setupurl} ) {
	for my $i (keys %$vars) {
	    $setup{setupurl} =~ s/%$i%/$vars->{$i}/g;
	}
	chomp(my @data = `wget -q -O- '$setup{setupurl}'`);
	for my $i (@data) {
	    for ($i) {
		/^\s*([^:]*[^\s:])\s*:\s*(.*[^\s])?\s*$/ && do {
		    $setup{$1} = $2?$2:"";
		}
	    }
	}
    }

    return (%setup);
}

#------------------------------------------------------------------------------
# Get local host information:
# Get hardware address, IP address and domain information.  These
# values may be used in S-terminal's "setupurl".  (See configuration
# subroutine.)
# Returned values are:
#  hwaddr     The computer's hardware (MAC) address
#  ipaddr     The computer's current IP address
#  ipdomain   The computer's IP domain, determined by doing a DNS lookup
#             on the IP address, then taking everything after the first
#             dot.  E.g., "my.host.mydomain.com" would have ipdomain set to
#             "host.mydomain.com".
#  ipdomain2  The same, but taking off both the first two elements of 
#             the host name.  E.g., "my.host.mydomain.com" would have
#             ipdomain2 set to "mydomain.com".
#------------------------------------------------------------------------------
sub LocalInfo {
    my %vars = ();
    my @data = ();
    
    # Get MAC address and IP address of eth0:
    chomp(@data = `/sbin/ifconfig eth0`);
    ($vars{hwaddr},$vars{ipaddr}) = grep {/(HWaddr|inet addr:)/} @data;
    $vars{hwaddr} = (split(/\s+/,$vars{hwaddr}))[4];
    $vars{hwaddr} =~ s/://g;
    $vars{ipaddr} = (split(/[\s:]+/,$vars{ipaddr}))[3];
    
    # Get IP domain name(s):
    chomp(@data = `host $vars{ipaddr}`);
    ($vars{ipdomain}) = grep {/(Name:|name)/} @data;
    @data = split(/\s+/,$vars{ipdomain});
    $vars{ipdomain} = $data[$#data];
    $vars{ipdomain} =~ s/^[^\.]+\.//;
    $vars{ipdomain} =~ s/\.$//;
    $vars{ipdomain2} = $vars{ipdomain};
    $vars{ipdomain2} =~ s/^[^\.]+\.//;

    return (%vars);
}

1;
